#!/bin/bash 
# projectid: 45991
# date: 2023-07-03

script_name=$(basename $0)
PWD=/etc/cpeagent/upgrade/${script_name%.*}

ret_success=0
ret_success_restart=1
ret_success_reboot=2
ret_fail_roolback=6
ret_fail_others=7

need_reboot=0
need_restartcpe=0


loacl_kernelversion=$(uname -r | awk -F "-g" '{print $1}')
upgrade_kernelversion="4.19.26-00812"

download_url="https://upgrade.sdwancloud.com/patch/LW2x08/depends/LW2508-DRIVER-0.0.2.tar.gz"
download_file="LW2508-DRIVER-0.0.2.tar.gz"
origin_md5_upgradefile="2b7b2e9ee233eab00c5af34a96184ab4"

update_config_xml_md5=90220cf0a19c7585cafec831848d8da0
update_policy_xml_md5=2c56a617f821cab6fe0305a2b63f5dcb

upgrade_path=/etc/cpeagent/upgrade/${script_name%.*}
logfile="/var/log/update.log"


########################################################################################################################################
#PATCH THIRD 0.0.7 #####################################################################################################################
########################################################################################################################################

update_config_xml()
{
    if [ -f /etc/fmc/config/config.xml ]; then
        origin_config_xml_md5=$(md5sum /etc/fmc/config/config.xml | awk '{print $1}')
    else
        echo "system no config.xml"
        exit $ret_fail_others
    fi

}

update_policy_xml()
{
    if [ -f /etc/fmc/config/policy.xml ];then
        origin_policy_xml_md5=$(md5sum /etc/fmc/config/policy.xml | awk '{print $1}')
    else
        echo "system no policy.xml"
        exit $ret_fail_others
    fi
}

install_allxml()
{
    update_policy_xml
    update_config_xml
    if [ "${update_config_xml_md5}" != "${origin_config_xml_md5}" -o "${update_policy_xml_md5}" != "${origin_policy_xml_md5}" ]; then
        mv /etc/fmc/config/config.xml /etc/fmc/config/config.xml.bak && mv /etc/fmc/config/policy.xml /etc/fmc/config/policy.xml.bak
        if [ $? -eq 0 ]; then 
            echo "backup config.xml policy.xml success"
            cp ${PWD}/files/config.xml /etc/fmc/config/config.xml -f && cp ${PWD}/files/policy.xml /etc/fmc/config/policy.xml -f 
            if [ $? -eq 0 ]; then
                need_reboot=1
                echo "update config.xml and policy.xml all success"
                return $ret_success
            else
                echo "update config.xml and policy.xml faild, need to rollback "
                exit $ret_fail_roolback 
            fi
        else
            echo "backup config.xml policy.xml failed "
            exit $ret_fail_others
        fi
    else
        echo "config.xml and policy.xml all already new file "
        return $ret_success
    fi
}

uninstall_xmlfiles()
{
    if [ -f /etc/fmc/config/config.xml.bak -a -f /etc/fmc/config/policy.xml.bak ]; then 
        mv /etc/fmc/config/config.xml.bak /etc/fmc/config/config.xml -f  && mv /etc/fmc/config/policy.xml.bak /etc/fmc/config/policy.xml -f 
        if [ $? -eq 0 ]; then 
            echo "roolback success "
            return $ret_success
        else
            echo "roolback failed "
            exit $ret_fail_others
        fi
    else
        echo " no need to roolback "
        exit $ret_fail_others
    fi
}

########################################################################################################################################
#PATCH DIRVER 0.0.2#####################################################################################################################
########################################################################################################################################

#下载及校验升级包
download_upgradefile()
{
    cd $PWD
    curl -O $download_url
    if [ $? -ne 0 ]; then
        echo "download upgradefile failed !"
        exit $ret_fail_others
    else
        echo "download upgradefile success !"
    fi

    download_md5_upgradefile=$(md5sum $PWD/$download_file | awk '{print $1}')
    if [ "$origin_md5_upgradefile" = "$download_md5_upgradefile" ]; then
        tar xf $download_file
        if [ $? -ne 0 ]; then
            echo "$download_file decompression failed !"
            exit $ret_fail_others
        else
            echo "$download_file decompression success !"
            find . -name $download_file -execdir rm -rf {} \;
            echo "clean $download_file success !"
        fi
    else
        echo  "md5sum is different, exit the upgrade process !"
        find . -name $download_file -execdir rm -rf {} \;
        echo "clean $download_file success !"
        exit $ret_fail_others
    fi

}


#判断根目录下是否有660M的空间，大于660M 继续升级流程，小于退出升级
check_disk_space()
{
    AvailableSpace=$(df | grep -w "\/" | awk '{print $4}')
    AvailableSpace=$(expr $AvailableSpace / 1024)
    if [ $AvailableSpace -lt 660 ]; then
        echo "AvailableSpace is less than 660M, upgrade is not recommended !"
        exit $ret_fail_others
    fi
    echo "AvailableSpace  is more than 660M, upgrade will continue !"
}


upgrade_boot()
{
    pushd /boot/ && find .  -maxdepth 1  -execdir rm -rf {} \; && popd
    tar xf  ${PWD}/bootpartition_LS_arm64_lts_4.19.tgz  -C /boot/
    if [ $? -eq 0 ]; then
        echo "upgrade boot success !"
    else
        echo "upgrade boot failed ,need to roolback !"
        exit $ret_fail_roolback
    fi
}

backup_boot()
{
    pushd /boot/
    tar cfz /lib/modules/boot_${loacl_kernelversion}_file.tar.gz *
    if [ $? -eq 0 ]; then
        echo "backup boot success !"
    else
        echo "backup boot failed ! "
        exit $ret_fail_others
    fi
    popd
}


upgrade_kernel()
{
    tar xf  ${PWD}/lib_modules_LS_arm64_4.19.26-00812-g1eed2e3da8ca.tgz  -C /lib/modules/
    if [ $? -eq 0 ]; then
        echo "upgrade kernel success !"
    else
        echo "upgrade kernel failed ,need to roolbak !"
        exit $ret_fail_roolback
    fi
}

clean_upgradefile()
{
    find . -name lib_modules_LS_arm64_4.19.26-00812-g1eed2e3da8ca.tgz -execdir rm -f {} \;
    echo "clean lib_modules_LS_arm64_4.19.26-00812-g1eed2e3da8ca.tgz  success !"
    find . -name bootpartition_LS_arm64_lts_4.19.tgz -execdir rm -f {} \;
    echo "clean bootpartition_LS_arm64_lts_4.19.tgz  success !"
}

install_kernel()
{
    if [ $loacl_kernelversion = $upgrade_kernelversion ]; then
        echo "The loacl_kernelversion is $loacl_kernelversion equal to the version to be upgraded, no need to upgrade !"
        return $ret_success
    else
        result_version=$(echo -e "$loacl_kernelversion\n$upgrade_kernelversion" | sort -V | head -n1)
        if [ $result_version = $loacl_kernelversion ]; then
            check_disk_space
            download_upgradefile
            backup_boot
            upgrade_boot
            upgrade_kernel
            sync
            clean_upgradefile
            echo  "upgrade boot and kernel all success !"
            need_reboot=1;
            return $ret_success
        elif [ $result_version = $upgrade_kernelversion ]; then 
            echo "The loacl_kernelversion is $loacl_kernelversion greater than the version to be upgraded, no need to upgrade !"
            return $ret_success
        fi
    fi
}

uninstall_kernel()
{
    if [ $loacl_kernelversion != $upgrade_kernelversion ]; then
        #回退boot
        if [ -f /lib/modules/boot_${loacl_kernelversion}_file.tar.gz ]; then
            pushd /boot/ && find .  -maxdepth 1  -execdir rm -rf {} \; && popd
            tar xf /lib/modules/boot_${loacl_kernelversion}_file.tar.gz -C /boot/
            if [ $? -eq 0 ]; then
                pushd /lib/modules/ && find . -name boot_${loacl_kernelversion}_file.tar.gz -execdir rm -rf {} \; && popd
                echo "roolback boot success !"
            else
                echo "roolback boot failed !"
                return $ret_fail_others
            fi
        else
            echo "No need to rollback boot !"
        fi
        #回退kernel
        if [ -d /lib/modules/$upgrade_kernelversion ]; then
            pushd /lib/modules/ && find . -name $upgrade_kernelversion -execdir rm -rf {} \; && popd
            echo  "roobalk kernel success !"
        else
            echo "No need to rollback kernel !"
            return $ret_fail_others
        fi
    else
        echo "No need to rollback kernel !"
        return $ret_fail_others
    fi
}

########################################################################################################################################
#PATCH THIRD 0.0.2######################################################################################################################
########################################################################################################################################

log_exit() #details #ret
{
    echo `date` "$script_name upgrade: ${1} exit $2" >> $logfile
    exit ${2}
}

#update_sw <rpm_path> <rpm_ver>
update_sw()
{
    rpm=`rpm -qa | grep $2`
    [ "$rpm" == "$2" ] && return 0
    [ ! -f $1 ] && log_exit "missing $1" $ret_fail_others
    yum install -y $1 >> $logfile 2>&1 || log_exit "yum install $1 failed" $ret_fail_others
}

#update_sw-libipsec <rpm_path> <rpm_ver>
update_sw_libipsec()
{
    rpm=`rpm -qa | grep $2`
    [ "$rpm" == "$2" ] && return 0
    [ ! -f $1 ] && log_exit "missing $1" $ret_fail_others
    yum install -y $1 >> $logfile 2>&1 || log_exit "yum install $1 failed" $ret_fail_others
}

#update_trousers <rpm_path> <rpm_ver>
update_trousers()
{
    rpm=`rpm -qa | grep $2`
    [ "$rpm" == "$2" ] && return 0
    [ ! -f $1 ] && log_exit "missing $1" $ret_fail_others
    yum install -y $1 >> $logfile 2>&1 || log_exit "yum install $1 failed" $ret_fail_others
}

install_strongwan()
{
    localstrongswan_version=$(rpm -qa  strongswan)
    if [ -z $localstrongswan_version ]; then 
        update_trousers "${upgrade_path}/packages/trousers-0.3.14-2.el7.aarch64.rpm"  "trousers-0.3.14-2.el7.aarch64"
        update_sw_libipsec "${upgrade_path}/packages/strongswan-libipsec-5.7.2-1.el7.aarch64.rpm" "strongswan-libipsec-5.7.2-1.el7.aarch64"
        update_sw "${upgrade_path}/packages/strongswan-5.7.2-1.el7.aarch64.rpm" "strongswan-5.7.2-1.el7.aarch64"
        cp -rf ${upgrade_path}/strongswan/* /usr/lib64/strongswan/
        echo  "install strongswan success " 
        need_restartcpe=1
        return $ret_success_restart
    else
        echo "strongswan has installed , no need to install strongswan"
        return $ret_success
    fi
}

check_error_and_exit()
{
    if [ ! $1 -eq 0 ];then
        echo "$2"
        exit  7
    fi
}


httpd_conf()
{  
    file="/etc/httpd/conf/httpd.conf"
    content1="ServerSignature Off"
    content2="ServerTokens Prod"

    if grep -q "$content1" "$file" && grep -q "$content2" "$file"; then
        echo "The lines 'ServerSignature Off' and 'ServerTokens Prod' both exist in the file."
    else
        echo "modify httpd conf start"
        if ! grep -qF "$content1" "$file";then
            sed -i '$a\ServerSignature Off' /etc/httpd/conf/httpd.conf
            ret=$?
            if [ $ret -eq 0 ]; then
                echo "Adding ServerSignature Off to httpd conf succeed"
            else
                check_error_and_exit $ret "Adding ServerSignature Off to httpd conf faild"
            fi
        fi

        if ! grep -qF "$content2" "$file"; then
            sed -i '$a\ServerTokens Prod' /etc/httpd/conf/httpd.conf
            ret=$?
            if [ $ret -eq 0 ]; then
                echo "Adding ServerTokens Prod to httpd conf succeed"
            else
                check_error_and_exit $ret "Adding ServerTokens Prod to httpd conf faild"
            fi
        fi  
        echo "modify httpd conf success"

        process_info=$(lsof -i :20080 | awk 'NR==2 {print $1, $2}')
        process_name=$(echo "$process_info" | awk '{print $1}')
        if [ "$process_name" == "lighttpd" ]; then
            process_id=$(echo "$process_info" | awk '{print $2}')
            kill -9 $process_id && systemctl restart httpd
            check_error_and_exit $? "http restart faild"
            echo "###################################################################"
            echo "restart httpd success!"
            echo "###################################################################" 
        else
            systemctl restart httpd
            check_error_and_exit $? "http restart faild"
            echo "###################################################################"
            echo "restart httpd success!"
            echo "###################################################################" 
        fi 
    fi  
}

php_conf()
{
    if ! grep -qE "^expose_php = Off$" "/etc/php.ini"; then
        echo "modify php conf start"
        sed -i 's/expose_php = On/expose_php = Off/' /etc/php.ini
        ret=$?
        if [[ $ret -eq 0 ]]; then
            echo "modify php conf success"
        else
            echo "modify php conf failed"
        fi
    else
        echo "The PHP configuration already exists in the file."
    fi
}
######################################################################################################################

install()
{
     cpe_version=$(cat /etc/cpeagent/RPM_version)
     cpe_num=${cpe_version:0:3}
     if (( $(echo "$cpe_num > 5.8" | bc -l) )) ||  (( $(echo "$cpe_num < 5.0" | bc -l) )); then
         echo "the cpe version is $cpe_version, this patch not support !"
         exit 7
     fi
    install_kernel
    install_allxml
    install_strongwan

    if (( $(echo "$cpe_num <= 5.5" | bc -l) )); then
        php_conf
        httpd_conf
    fi

    if [ $need_reboot -eq 1 ]; then
        exit $ret_success_reboot
    elif [ $need_restartcpe -eq 1 ]; then
        exit $ret_success_restart
    else
        exit $ret_success
    fi
}

uninstall()
{
    uninstall_kernel
    uninstall_xmlfiles
}

if [ "$1" == "install" ];then
    install
elif [ "$1" == "uninstall" ]; then
    uninstall
else
    echo  "invalid args" $ret_fail_others
fi





